@@ -24,13 +24,14 @@ sub readopts {
 	if(($self->has_pass && $self->pass < 2) || !$self->has_pass) {
 		die "refusing to overwrite file " . $self->url . "!\n" if (-f $self->url);
 
-		print "Writing " . $self->url . " with content:\n";
-		open CONCAT, ">" . $self->url;
+		my $content = "ffconcat version 1.0\n\n";
 		foreach my $component(@{$self->components}) {
 			my $input = $component->url;
-			print "file '$input'\n";
-			print CONCAT "file '$input'\n";
+			$content .= "file '$input'\n";
 		}
+		print "Writing " . $self->url . " with content:\n$content\n";
+		open CONCAT, ">" . $self->url;
+		print CONCAT $content;
 		close CONCAT;
 	}
 
